#!/usr/bin/env python

#
# Copyright 2013 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#
# Last changed: 2013-08-26 by FNA
#

#import bananaconst

####################
# Event class      #
# generic class    #
# to store history #
# events           #
####################

class Event:

    # instance variables

    def __init__(self, tile):
        self.tile = tile

    def destroy():
        pass

    # diagnostic
    def dump(self):
        str = ""
        str += "Event-do called\n"
        str += "event is of type: " + self.eventtype + "\n"
        str += "Start is: "
        str += repr(self.start) + "\n"
        str += "End is: "
        str += repr(self.end) + "\n"
        return str


###################################
# Move class -- subclass of Event #
# stores tile move actions        #
###################################
class Move(Event):

    def __init__(self, tile, x1, y1, x2, y2):
        Event.__init__(self, tile)
        self.eventtype = "move"
        d1 = { 'x': x1,
               'y': y1,
             }
        d2 = { 'x': x2,
               'y': y2,
             }
        self.start = d1
        self.end = d2

        def destroy():
            pass

    def do(self, operation):
        if operation == "undo":
            # set tile position to start value
            self.tile.x = self.start['x']
            self.tile.y = self.start['y']
        elif operation == "redo":
            # set tile position to end value
            self.tile.x = self.end['x']
            self.tile.y = self.end['y']

#####################################
# Select class -- subclass of Event #
# stores tile selection actoins     #
#####################################
class Select(Event):
    def __init__(self, tile, list1, list2):
        Event.__init__(self, tile)
        self.eventtype = "select"
        d1 = { 'selected' : list1,
             }
        d2 = { 'selected' : list2,
             }
        self.start = d1
        self.end = d2

    def do(self, operation):
        # need a better variable name than self.tile!!!
        banana = self.tile
        if operation == "undo":
            # set tile position to start value
            banana.selected = self.start['selected']
        elif operation == "redo":
            # set tile position to end value
            banana.selected = self.end['selected']
        for tile in banana.tiles:
            if tile in banana.selected:
                tile.selected = True
            else:
                tile.selected = False
        #print "Select-do called"
        #print "event is of type: " + self.eventtype
        #print "Start is: ",
        #print self.start
        #print "End is: ",
        #print self.end

def main():
    print __name__+ " is not meant to be run on its own."
    print "Testing Event class"

    print "Creating a Move subclass"
    m = Move(0, 0, 100, 100)

    print "Creating a Select subclass"
    s = Select(['a', 'b', 'c'], ['x', 'y', 'z'])

    print "calling m.do('do')"
    m.do('do')

    print "calling s.do('do')"
    s.do('do')

    return 0

if __name__ == "__main__":
    main()
