#!/usr/bin/env python

#
# Copyright 2013 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#
# Last changed: 2013-08-26 by FNA
#

# Create a data structure to store undo and redo information on stuff
# The bananaevent class contains the details of what to do/undo/redo

#################
# History Class #
#################

class History:

    # methods
    def __init__(self):
        # instance variables
        # 'undo' and 'redo' queues
        self.q = dict();
        self.q['undo'] = []
        self.q['redo'] = []

    def destroy():
        pass


    #######################
    # record a new action #
    #######################
    def do(self, eventlist):
        self.q['undo'].append(eventlist)
        # all other changes on the redo queue are lost
        self.q['redo'] = []

    ##################
    # undo an action #
    ##################
    def undo(self):
        self._q_do('undo', 'redo', 'undo')

    ##################
    # redo an action #
    ##################
    def redo(self):
        self._q_do('redo', 'undo', 'redo')

    ######################################################
    # return a list of all the actions in the undo queue #
    ######################################################
    def show(self):
        length = len(self.q['undo'])
        if length == 0:
            return None

        return self.q['undo']

    ##################################################
    # return a string of all the events in the queue #
    ##################################################
    def list(self):
        string = ""
        for el in self.show():
            for e in el:
                string += e.dump()
        return string


    ####################################
    # internal function:               #
    # undo/redo an action, moving an   #
    # event from the src q to the dest #
    # q, and performing the operation, #
    # i.e., do the start action (undo) #
    # or the end action (redo) of the  #
    # event.                           #
    ####################################

    def _q_do(self, srcq, destq, operation):
        if len(self.q[srcq]) == 0:
            return "Empty queue, nothing to undo/redo."

        # take the event off the source queue
        eventlist = self.q[srcq].pop()

        # perform the action of the event
        for event in eventlist:
            event.do(operation)

        # add the event to the destination q
        self.q[destq].append(eventlist)

def main():

    from bananaevent import Move, Select 

    print __name__+ " is not meant to be run on its own."
    print "testing history"

    print "Creating a history queue"
    h = History()

    print "Creating a Move event"
    m = Move(0, 0, 100, 100)

    print "Creating a Select event"
    s = Select(['a', 'b', 'c'], ['x', 'y', 'z'])

    print "Adding move to the queue"
    h.do([m])

    print "Adding select to the queue"
    h.do([s])
    
    print "History queue is: "
    l = h.show()
    for el in l:
        for e in el:
            print e.dump()

    print "undoing thing"
    h.undo()

    print "History queue is: "
    l = h.show()
    for el in l:
        for e in el:
            print e.dump()

    print "redoing thing"
    h.redo()

    print "History queue is: "
    l = h.show()
    for el in l:
        for e in el:
            print e.dump()


    return 0


if __name__ == "__main__":
    main()
