#!/usr/bin/env python

#
# Copyright 2012 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#
# Last changed: 2012-02-02 by FNA
#

import pygtk
pygtk.require('2.0')
import gtk

import banana
import bananaconst

class Menu:

    # instance variables (global thingy)
    obj = 0

    # methods

    # quit method
    def destroy(self, widget):
        # do nothing
        return

    
    # displatch the proper function based on what menu was selected
    def handle_menu(self, widget, string):
        if string == "file.new": 
            banana.do_dialog(self.obj, 
                             "Really reset?", "Really start a new game?", 
                             self.obj.reset)
        elif string == "file.prop": properties(self.obj)
        elif string == "file.quit": 
            banana.quit_dialog(self.obj)
        elif string == "prop.spellcheck": banana.prop.togglespellcheck(self.obj)
        elif string == "edit.undo":
            self.obj.history.undo()
            self.obj.redraw_cb()
        elif string == "edit.redo":
            self.obj.history.redo()
            self.obj.redraw_cb()
        elif string == "select.all":
            self.obj.selected = []
            for tile in self.obj.tiles:
                self.obj.selected.append(tile)
                tile.selected = 1
            self.obj.redraw_cb()
            # everyone is selected, we won't recalculate neighbors elsewhere
            self.obj.allselected = 1
        elif string == "select.none": 
            self.obj.unselect_all()
        elif string == "size.small":  self.obj.labels['window'].resize(500, 500)
        elif string == "size.medium": self.obj.labels['window'].resize(600, 600)
        elif string == "size.large":  self.obj.labels['window'].resize(700, 700)
        elif string == "help.about": about(self.obj)
        elif string == "help.help": help(self.obj)

    # start method
    def __init__(self, vbox, obj, accel_group):

        # save this for later
        self.obj = obj

        menubar = gtk.MenuBar()
        vbox.pack_start(menubar, False, False, 2)

        # File menu
        file_item = gtk.MenuItem("_File")
        menubar.append(file_item)

        filemenu = gtk.Menu()

        openfile = gtk.MenuItem("_New Game")
        sepfile  = gtk.SeparatorMenuItem()
        propfile = gtk.MenuItem("_Properties")
        sepfile1  = gtk.SeparatorMenuItem()
        quitfile = gtk.MenuItem("_Quit")

        filemenu.append(openfile)
        filemenu.append(sepfile)
        filemenu.append(propfile)
        filemenu.append(sepfile1)
        filemenu.append(quitfile)

        openfile.connect("activate", self.handle_menu, "file.new")
        propfile.connect("activate", self.handle_menu, "file.prop")
        quitfile.connect("activate", self.handle_menu, "file.quit")

        # accelerator control-Q to quit
        quitfile.add_accelerator("activate", accel_group, 
                                 ord('Q'), gtk.gdk.CONTROL_MASK, 
                                 gtk.ACCEL_VISIBLE)

        file_item.set_submenu(filemenu)

        # Edit menu
        edit_item = gtk.MenuItem("_Edit")
        menubar.append(edit_item)

        editmenu = gtk.Menu()

        undoedit = gtk.MenuItem("_Undo")
        redoedit = gtk.MenuItem("_Redo")

        editmenu.append(undoedit)
        editmenu.append(redoedit)

        undoedit.connect("activate", self.handle_menu, "edit.undo")
        redoedit.connect("activate", self.handle_menu, "edit.redo")

        # accelerator control-z to undo
        undoedit.add_accelerator("activate", accel_group, 
                                 ord('z'), gtk.gdk.CONTROL_MASK, 
                                 gtk.ACCEL_VISIBLE)
        # accelerator control-y to redo
        redoedit.add_accelerator("activate", accel_group, 
                                 ord('y'), gtk.gdk.CONTROL_MASK, 
                                 gtk.ACCEL_VISIBLE)

        edit_item.set_submenu(editmenu)


        # Select menu
        select_item = gtk.MenuItem("_Select")
        menubar.append(select_item)

        selectmenu = gtk.Menu()
        selectall = gtk.MenuItem("_Select All")
        selectnone = gtk.MenuItem("_Deselect All")

        selectmenu.append(selectall)
        selectmenu.append(selectnone)

        selectall.connect("activate", self.handle_menu,"select.all")
        selectnone.connect("activate", self.handle_menu,"select.none")

        # accelerator control-a to select all
        selectall.add_accelerator("activate", accel_group, 
                                  ord('a'), gtk.gdk.CONTROL_MASK, 
                                  gtk.ACCEL_VISIBLE)

        # accelerator control-d to deselect all
        selectnone.add_accelerator("activate", accel_group, 
                                   ord('d'), gtk.gdk.CONTROL_MASK, 
                                   gtk.ACCEL_VISIBLE)

        select_item.set_submenu(selectmenu)

        # Size menu
        size_item = gtk.MenuItem("_Size")
        menubar.append(size_item)

        sizemenu = gtk.Menu()

        sizesmall  = gtk.MenuItem("_Small") 
        sizemedium = gtk.MenuItem("_Medium") 
        sizelarge  = gtk.MenuItem("_Large") 

        sizemenu.append(sizesmall)
        sizemenu.append(sizemedium)
        sizemenu.append(sizelarge)

        sizesmall.connect("activate", self.handle_menu, "size.small")
        sizemedium.connect("activate", self.handle_menu, "size.medium")
        sizelarge.connect("activate", self.handle_menu, "size.large")

        # accelerator control-1 for small
        sizesmall.add_accelerator("activate", accel_group, 
                                  ord('1'), gtk.gdk.CONTROL_MASK, 
                                  gtk.ACCEL_VISIBLE)
        # accelerator control-2 for medium
        sizemedium.add_accelerator("activate", accel_group, 
                                   ord('2'), gtk.gdk.CONTROL_MASK, 
                                   gtk.ACCEL_VISIBLE)
        # accelerator control-3 for large
        sizelarge.add_accelerator("activate", accel_group, 
                                  ord('3'), gtk.gdk.CONTROL_MASK, 
                                  gtk.ACCEL_VISIBLE)

        size_item.set_submenu(sizemenu)

        # Help menu
        help_item = gtk.MenuItem("_Help")
        menubar.append(help_item)

        helpmenu = gtk.Menu()

        helpabout = gtk.MenuItem("_About") 
        helphelp = gtk.MenuItem("_Help") 

        helpmenu.append(helpabout)
        helpmenu.append(helphelp)

        helpabout.connect("activate", self.handle_menu, "help.about")
        helphelp.connect("activate", self.handle_menu, "help.help")

        # accelerator control-H to help
        helphelp.add_accelerator("activate", accel_group, 
                                 ord('H'), gtk.gdk.CONTROL_MASK, 
                                 gtk.ACCEL_VISIBLE)

        help_item.set_submenu(helpmenu)

        # menu bar
        menubar.show_all()

        return

def about(obj):
    dialog = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT, 
                               gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "")

    dialog.set_title("About " + bananaconst.name)
    dialog.set_markup(bananaconst.abouttext)

    dialog.show_all()
    dialog.connect('response', lambda x,w: dialog.destroy())
    return

def help(dmobj):
    
    dialog = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT, 
                               gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "")

    # make a scroll bar
    scroll = gtk.ScrolledWindow()
    scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
    scroll.set_size_request(400, 300)
    dialog.get_content_area().add(scroll)

    dialog.set_title("Help on " + bananaconst.name)
    dialog.set_markup(bananaconst.name + " version " + 
                      bananaconst.version + " Online Help")

    label = gtk.Label()
    label.set_markup(bananaconst.helptext)

    label.set_alignment(0,0)
    scroll.add_with_viewport(label)

    scroll.connect('key_press_event', key_press_handler)

    dialog.show_all()
    dialog.connect('response', lambda x,w: dialog.destroy())


def key_press_handler(w, event):
    keyname = gtk.gdk.keyval_name(event.keyval)

    # make it so space or return when the scrollbar has focus
    # generates a response equal to hitting the "OK" button or 
    # dismissing the dialog

    if (keyname == "Return" or keyname == "space"):
        # get the dialog widget the great-grand-parent of the scrollbar
        w.get_parent().get_parent().response(1)


# pop up a properties dialog window
def properties(obj):

    dialog = gtk.Dialog("Set properties", None, 
                        gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                         gtk.STOCK_OK, gtk.RESPONSE_OK))

    # tile file name specification
    frame = gtk.Frame("Tile File")
    dialog.vbox.pack_start(frame, True, True, 0)

    vbox = gtk.VBox(False, 0)
    frame.add(vbox)

    hbox = gtk.HBox(False, 0)
    vbox.pack_start(hbox, False, False, 0)

    label = gtk.Label(bananaconst.name + " tile file: ")
    hbox.pack_start(label, False, False, 0)

    tilefile = gtk.Entry()
    tilefile.set_text(bananaconst.tilefile)
    hbox.pack_start(tilefile, False, False, 0)

    # spelling check configuration
    frame = gtk.Frame("Spelling Check")
    dialog.vbox.pack_start(frame, True, True, 0)
    vbox = gtk.VBox(False, 0)
    frame.add(vbox)

    hbox = gtk.HBox(False, 0)
    vbox.pack_start(hbox, False, False, 0)

    # radio button
    radio1 = gtk.RadioButton(None, "No checking")
#    radio1.connect("toggled", self.callback, "button1")
    hbox.pack_start(radio1, True, True, 0)

    radio2 = gtk.RadioButton(radio1, "Loose checking")
#    radio2.connect("toggled", self.callback, "button2")
    hbox.pack_start(radio2, True, True, 0)

    radio3 = gtk.RadioButton(radio2, "Strict checking")
#    radio3.connect("toggled", self.callback, "button3")
    hbox.pack_start(radio3, True, True, 0)

    # pick which radio button is the default based on
    # the Banana.spellmode variable in the main object
    (radio1, radio2, radio3)[obj.spellmode].set_active(True)

    # file name 
    hbox = gtk.HBox(False, 0)
    vbox.pack_start(hbox, False, False, 0)

    label = gtk.Label(bananaconst.name + " spelling word file: ")
    hbox.pack_start(label, False, False, 0)

    spellingfile = gtk.Entry()
    spellingfile.set_text(bananaconst.spellingfile)
    hbox.pack_start(spellingfile, False, False, 0)

    # grid box
    frame = gtk.Frame("Align Tiles to Grid")
    dialog.vbox.pack_start(frame, True, True, 0)
    vbox = gtk.VBox(False, 0)
    frame.add(vbox)

    hbox = gtk.HBox(False, 0)
    vbox.pack_start(hbox, False, False, 0)

    # radio button
    radio4 = gtk.RadioButton(None, "No")
#    radio1.connect("toggled", self.callback, "button1")
    hbox.pack_start(radio4, True, True, 0)

    radio5 = gtk.RadioButton(radio4, "Yes")
#    radio2.connect("toggled", self.callback, "button2")
    hbox.pack_start(radio5, True, True, 0)

    # pick which radio button is the default based on
    # the Banana.spellmode variable in the main object
    (radio4, radio5)[obj.grid].set_active(True)



    dialog.show_all()
    response = dialog.run()

    # set default
    needredraw = 0

    if response == gtk.RESPONSE_OK:
        # set things
        bananaconst.tilefile = tilefile.get_text()
        bananaconst.spellingfile = spellingfile.get_text()

        # set the spelling check mode
        if radio1.get_active():   mode = 0
        elif radio2.get_active(): mode = 1
        elif radio3.get_active(): mode = 2

        obj.spellmode = mode

        # set the grid mode
        if radio4.get_active():   grid = 0
        elif radio5.get_active(): grid = 1

        # figure out if it changed
        if (radio4, radio5)[obj.grid]:
            needredraw = 1
        else:
            needredraw = 0

        obj.grid = grid

    dialog.destroy()

    # redraw the grid if we toggled it
    if needredraw: obj.redraw_cb()

    return


def main():
    print "This function is not intended to run by itself, use banana.py instead"
    return 0

if __name__ == "__main__":

    main()


# end


