#!/usr/bin/python

#
# Copyright 2012 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#
# Last changed: 2012-02-02 by FNA
#

import pygtk
pygtk.require('2.0')
import gtk

import bananaconst

class Tile:

    # instance variables

    letter = ''
    widget = 0
    x = 0
    y = 0
    width = 0
    height = 0

    gc = 0
    whitegc = 0
    selectedgc = 0
    redgc = 0

    offsetx = 0
    offsety = 0

    # track the original position when dragging
    origx = 0
    origy = 0
    selected = 0

    # links to its 4 neighbors (if any) 
    top = 0
    bottom = 0
    left = 0
    right = 0
    widget = 0

    misspelled = 0

    # methods

    def destroy(self, widget):
        pass

    # start method
    def __init__(self, letter, area, x, y, width, height):
        self.letter = letter
        self.widget = area
        self.x = x
        self.y = y
        self.width = width
        self.height = height

        return

    ##############################################
    # report if an (x,y) coordinate falls within #
    # this tile's position                       #
    ##############################################
    def within(self, x, y):
        if (x < self.x or x > (self.x + self.width) or
            y < self.y or y > (self.y + self.height)):
            return False
        self.offsetx = self.x - x
        self.offsety = self.y - y
        return True

    ####################################
    # report if the tile is inside the #
    # (x1, y1), (x2, y2) rectangle     #
    ####################################
    def inside(self, x1, y1, x2, y2):

        # save this for later, as these are the current
        # mouse position
        savex = x2
        savey = y2

        # make rectangle so x1 < x2, and y1 < y2
        if x1 > x2:
            # swap if out of order
            temp = x1
            x1 = x2
            x2 = temp
        if y1 > y2:
            # swap if out of order
            temp = y1
            y1 = y2
            y2 = temp
            
        # check against both of the x bounds
        if (self.x + self.width) < x1:
            return False
        if self.x > x2:
            return False
        # check against both of the y bounds
        if (self.y + self.height) < y1:
            return False
        if self.y > y2:
            return False

        # otherwise, it's within, so save the offsets and return true
#        self.offsetx = self.x - savex
#        self.offsety = self.y - savey
        return True


    ####################################
    # draw the tile on the DrawingArea #
    ####################################
    def draw(self, banana):
        if self.selected:
            tempgc = banana.selectedgc
        else:
            tempgc = banana.whitegc

        if self.misspelled:
            textgc = banana.redgc
        else:
            textgc = banana.gc

        self.widget.window.draw_rectangle(banana.gc, False, int(self.x), int(self.y), self.width, self.height)
        self.widget.window.draw_rectangle(tempgc, True, int(self.x+2), int(self.y+2), self.width-3, self.height-3)
        banana.pangolayout.set_text(self.letter)
        self.widget.window.draw_layout(textgc, int(self.x + 10), int(self.y + 5), banana.pangolayout)
        return

##### end Class Tile #######


####################################
# Simple logic check for comparing #
# north/south/east/west neighbors. #
# Logic is in how the variables    #
# are marshalled in the caller.    #
####################################
def nsewCheck (x1, x2, x3, x4):

    if x1 > x2:
        return False
    if x3 > x4:
        return False
    return True


##############################################
# check for neighbors                        #
#                                            #
# For N/S/E/W neighbor, other tile must be   #
# aligned in one direction (x or y) within   #
# a an error bound (height or width) and one #
# tile height or width more or less than     #
# centertile in the other direction          #
##############################################
def neighborCheck(centertile, othertile, direction):
    if direction == 'north':
        # tiles must be aligned within width/2 in x direction and above
        # centertile within height/2 in y direction
        return nsewCheck (abs(centertile.x - othertile.x),
                          centertile.width/2, 
                          abs(centertile.y - (othertile.y + othertile.height)),
                          centertile.height/2)

    if direction == 'south':
        # tiles must be aligned within width/2 in x direction and below
        # centertile within height/2 in y direction
        return nsewCheck (abs(centertile.x - othertile.x),
                          centertile.width/2, 
                          abs(centertile.y - (othertile.y - othertile.height)),
                          centertile.height/2)

    if direction == 'east':
        # tiles must be aligned within height/2 in y direction and to the
        # right of centertile within width/2 in x direction
        return nsewCheck (abs(centertile.y - othertile.y),
                          centertile.height/2, 
                          abs(centertile.x - (othertile.x - othertile.width)),
                          centertile.width/2)

    if direction == 'west':
        # tiles must be aligned within height/2 in y direction and to the 
        # left of centertile within width/2 in the x direction
        return nsewCheck (abs(centertile.y - othertile.y),
                          centertile.height/2, 
                          abs(centertile.x - (othertile.x + othertile.width)),
                          centertile.width/2)

    # default (e.g., if direction is illegal)
    return False

####################
# update neighbors #
####################
def tileNeighbors(mytile, tiles):

    # Clear old neighbor links, if they exist.
    # Since it's a doubly-linked list, we must 
    # clear the links pointing back to us
    if mytile.top:
        mytile.top.bottom = 0
        mytile.top = 0
    if mytile.bottom:
        mytile.bottom.top = 0
        mytile.bottom = 0
    if mytile.left:
        mytile.left.right = 0
        mytile.left = 0
    if mytile.right:
        mytile.right.left = 0
        mytile.right = 0


    for tile in tiles:
        if tile == mytile:
            # skip if it's mytile
            continue

        # check for north neighbor
        if neighborCheck(mytile, tile, 'north'):
#            print "north neighbor!"
            # set each others' north/south values
            mytile.top = tile
            tile.bottom = mytile
        # check for south neighbor
        if neighborCheck(mytile, tile, 'south'):
#            print "south neighbor!"
            # set each others' north/south values
            mytile.bottom = tile
            tile.top = mytile
        # check for east neighbor
        if neighborCheck(mytile, tile, 'east'):
#            print "east neighbor!"
            # set each others' east/west values
            mytile.right = tile
            tile.left = mytile
        # check for west neighbor
        if neighborCheck(mytile, tile, 'west'):
#            print "west neighbor!"
            # set each others' east/west values
            mytile.left = tile
            tile.right = mytile

    return


################################
# find words in tiles          #
#                              #
# Input: list of tiles         #
# Result: list of form         #
# (tile, word, "right"|"down") #
#                              #
################################
def findWords(tiles):

    results = []
    # associative arrays to track paths we've already traversed
    upskip = dict()
    leftskip = dict()
    
    upwords = []
    leftwords = []

    # go through all the tiles
    for tile in tiles:

        # clear misspelling flag
        tile.misspelled = 0

        if tile.top == 0 and tile.bottom == 0 and tile.left == 0 and tile.right :
            # it's a singleton letter, skip
            continue

        # check for vertical words
        if tile.top != 0 and tile not in upskip:
            temptile = tile
            while temptile.top != 0:
                # mark that we've traversed this path
                upskip[temptile] = 1

                # go up one tile
                temptile = temptile.top
                # skip if we've already traversed this path
                if temptile in upskip:
                    break

            # add tile to list if we haven't been here before
            if temptile not in upskip:
                upskip[temptile] = 1
                upwords.append(temptile)

        # check for horizontal words
        if tile.left != 0 and tile not in leftskip:
            temptile = tile
            while temptile.left != 0:
                # mark that we've traversed this path
                leftskip[temptile] = 1

                # go left up one tile
                temptile = temptile.left
                # skip if we've already traversed this path
                if temptile in leftskip:
                    break

            # add tile to list if we haven't been here before
            if temptile not in leftskip:
                leftskip[temptile] = 1
                leftwords.append(temptile)


    # now extract the words going down
    for tile in upwords:
        word = ''
        savetile = tile
        while tile.bottom != 0:
            word += tile.letter
            tile = tile.bottom
        word += tile.letter
        results.append((savetile, word, "down"))

    # now extract the words to the right
    for tile in leftwords:
        word = ''
        savetile = tile
        while tile.right != 0:
            word += tile.letter
            tile = tile.right
        word += tile.letter
        results.append((savetile, word, "right"))

    return results


            
def main():
    print "This is intended to be called by banana.py"
    return 0

if __name__ == "__main__":
    main()

    
# end
